@@ -6,6 +6,7 @@ from .structures import Intermediate, Resource
 
 def get_hash(path: Path, type: str = "sha256") -> str:
         
+    from xxhash import xxh3_64_hexdigest
     from hashlib import sha1, sha256, sha512
     from murmurhash2 import murmurhash2 as murmur2
 
@@ -17,6 +18,7 @@ def get_hash(path: Path, type: str = "sha256") -> str:
         case "sha1": hash = sha1(data).hexdigest()
         case "sha256": hash = sha256(data).hexdigest()
         case "sha512": hash = sha512(data).hexdigest()
+        case "xxhash": hash = xxh3_64_hexdigest(data)
         case "murmur2": 
             data = bytes([b for b in data if b not in (9, 10, 13, 32)])
             hash = murmur2(data, seed=1)
